package xhttp3.internal.connection;

import java.io.IOException;
import java.lang.ref.Reference;
import java.net.ConnectException;
import java.net.ProtocolException;
import java.net.Proxy;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.net.UnknownServiceException;
import java.net.Proxy.Type;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import xhttp3.Address;
import xhttp3.Connection;
import xhttp3.ConnectionSpec;
import xhttp3.Handshake;
import xhttp3.HttpUrl;
import xhttp3.OkHttpClient;
import xhttp3.Protocol;
import xhttp3.Request;
import xhttp3.Response;
import xhttp3.Route;
import xhttp3.internal.Util;
import xhttp3.internal.Version;
import xhttp3.internal.framed.ErrorCode;
import xhttp3.internal.framed.FramedConnection;
import xhttp3.internal.framed.FramedStream;
import xhttp3.internal.http.Http1xStream;
import xhttp3.internal.http.HttpHeaders;
import xhttp3.internal.platform.Platform;
import xio.BufferedSink;
import xio.BufferedSource;
import xio.Okio;
import xio.Source;

public final class RealConnection extends FramedConnection.Listener implements Connection {
   private final Route route;
   private Socket rawSocket;
   public Socket socket;
   private Handshake handshake;
   private Protocol protocol;
   public volatile FramedConnection framedConnection;
   public int successCount;
   public BufferedSource source;
   public BufferedSink sink;
   public int allocationLimit;
   public final List<Reference<StreamAllocation>> allocations;
   public boolean noNewStreams;
   public long idleAtNanos;

   public RealConnection(Route route) {
      RealConnection var10000 = this;
      RealConnection var10001 = this;
      RealConnection var10002 = this;
      super();
      ArrayList var2;
      var2 = new ArrayList.<init>();
      var10002.allocations = var2;
      var10001.idleAtNanos = Long.MAX_VALUE;
      var10000.route = route;
   }

   private void buildTunneledConnection(int connectTimeout, int readTimeout, int writeTimeout, ConnectionSpecSelector connectionSpecSelector) {
      Request var5;
      HttpUrl var6 = (var5 = this.createTunnelRequest()).url();
      int var7 = 0;

      byte var8;
      for(var8 = 21; ++var7 <= var8; this.source = null) {
         this.connectSocket(connectTimeout, readTimeout);
         if ((var5 = this.createTunnel(readTimeout, writeTimeout, var5, var6)) == null) {
            this.establishProtocol(readTimeout, writeTimeout, connectionSpecSelector);
            return;
         }

         Util.closeQuietly(this.rawSocket);
         this.rawSocket = null;
         this.sink = null;
      }

      throw new ProtocolException("Too many tunnel connections attempted: " + var8);
   }

   private void buildConnection(int connectTimeout, int readTimeout, int writeTimeout, ConnectionSpecSelector connectionSpecSelector) {
      this.connectSocket(connectTimeout, readTimeout);
      this.establishProtocol(readTimeout, writeTimeout, connectionSpecSelector);
   }

   private void connectSocket(int connectTimeout, int readTimeout) {
      Proxy var3;
      Proxy var10000 = var3 = this.route.proxy();
      Address var4 = this.route.address();
      Socket var6;
      if (var10000.type() != Type.DIRECT && var3.type() != Type.HTTP) {
         var6 = new Socket.<init>(var3);
      } else {
         var6 = var4.socketFactory().createSocket();
      }

      this.rawSocket = var6;
      var6.setSoTimeout(readTimeout);

      try {
         Platform.get().connectSocket(this.rawSocket, this.route.socketAddress(), connectTimeout);
      } catch (ConnectException var5) {
         throw new ConnectException("Failed to connect to " + this.route.socketAddress());
      }

      this.source = Okio.buffer(Okio.source(this.rawSocket));
      this.sink = Okio.buffer(Okio.sink(this.rawSocket));
   }

   private void establishProtocol(int readTimeout, int writeTimeout, ConnectionSpecSelector connectionSpecSelector) {
      if (this.route.address().sslSocketFactory() != null) {
         this.connectTls(readTimeout, writeTimeout, connectionSpecSelector);
      } else {
         this.protocol = Protocol.HTTP_1_1;
         this.socket = this.rawSocket;
      }

      Protocol readTimeout1;
      if ((readTimeout1 = this.protocol) != Protocol.SPDY_3 && readTimeout1 != Protocol.HTTP_2) {
         this.allocationLimit = 1;
      } else {
         RealConnection var10000 = this;
         RealConnection var10001 = this;
         this.socket.setSoTimeout(0);
         FramedConnection.Builder readTimeout2;
         FramedConnection.Builder var10002 = readTimeout2 = new FramedConnection.Builder;
         readTimeout2.<init>(true);
         Socket readTimeout3 = this.socket;
         String writeTimeout1 = this.route.address().url().host();
         BufferedSource connectionSpecSelector1 = this.source;
         BufferedSink var4 = this.sink;
         FramedConnection var5;
         FramedConnection var11 = var5 = var10002.socket(readTimeout3, writeTimeout1, connectionSpecSelector1, var4).protocol(this.protocol).listener(this).build();
         var11.start();
         var10001.allocationLimit = var11.maxConcurrentStreams();
         var10000.framedConnection = var5;
      }

   }

   private void connectTls(int param1, int param2, ConnectionSpecSelector param3) {
      // $FF: Couldn't be decompiled
   }

   private Request createTunnel(int readTimeout, int writeTimeout, Request tunnelRequest, HttpUrl url) {
      String url1 = "CONNECT " + Util.hostHeader(url, true) + " HTTP/1.1";

      while(true) {
         Http1xStream var5;
         Http1xStream var10000 = var5 = new Http1xStream;
         BufferedSource var6 = this.source;
         BufferedSink var7 = this.sink;
         var5.<init>((OkHttpClient)null, (StreamAllocation)null, var6, var7);
         TimeUnit var13;
         this.source.timeout().timeout((long)readTimeout, var13 = TimeUnit.MILLISECONDS);
         this.sink.timeout().timeout((long)writeTimeout, var13);
         var10000.writeRequest(tunnelRequest.headers(), url1);
         var10000.finishRequest();
         Response tunnelRequest1;
         long var14;
         if ((var14 = HttpHeaders.contentLength(tunnelRequest1 = var10000.readResponse().request(tunnelRequest).build())) == -1L) {
            var14 = 0L;
         }

         Source var10001 = var5.newFixedLengthSource(var14);
         Util.skipAll(var10001, Integer.MAX_VALUE, var13);
         var10001.close();
         int var11;
         if ((var11 = tunnelRequest1.code()) == 200) {
            if (this.source.buffer().exhausted() && this.sink.buffer().exhausted()) {
               return null;
            }

            throw new IOException("TLS tunnel buffered too many bytes!");
         }

         if (var11 != 407) {
            throw new IOException("Unexpected response code for CONNECT: " + tunnelRequest1.code());
         }

         Request var12;
         if ((var12 = this.route.address().proxyAuthenticator().authenticate(this.route, tunnelRequest1)) == null) {
            throw new IOException("Failed to authenticate with proxy");
         }

         if ("close".equalsIgnoreCase(tunnelRequest1.header("Connection"))) {
            return var12;
         }

         tunnelRequest = var12;
      }
   }

   private Request createTunnelRequest() {
      return (new Request.Builder()).url(this.route.address().url()).header("Host", Util.hostHeader(this.route.address().url(), true)).header("Proxy-Connection", "Keep-Alive").header("User-Agent", Version.userAgent()).build();
   }

   public void connect(int connectTimeout, int readTimeout, int writeTimeout, List<ConnectionSpec> connectionSpecs, boolean connectionRetryEnabled) {
      if (this.protocol != null) {
         throw new IllegalStateException("already connected");
      } else {
         RouteException var6 = null;
         ConnectionSpecSelector var7;
         var7 = new ConnectionSpecSelector.<init>(connectionSpecs);
         if (this.route.address().sslSocketFactory() == null) {
            if (!connectionSpecs.contains(ConnectionSpec.CLEARTEXT)) {
               UnknownServiceException var11;
               var11 = new UnknownServiceException.<init>("CLEARTEXT communication not enabled for client");
               throw new RouteException(var11);
            }

            String connectionSpecs1 = this.route.address().url().host();
            if (!Platform.get().isCleartextTrafficPermitted(connectionSpecs1)) {
               throw new RouteException(new UnknownServiceException("CLEARTEXT communication to " + connectionSpecs1 + " not permitted by network security policy"));
            }
         }

         while(this.protocol == null) {
            IOException var10000;
            label52: {
               boolean var14;
               boolean var10001;
               try {
                  var14 = this.route.requiresTunnel();
               } catch (IOException var10) {
                  var10000 = var10;
                  var10001 = false;
                  break label52;
               }

               if (var14) {
                  try {
                     this.buildTunneledConnection(connectTimeout, readTimeout, writeTimeout, var7);
                     continue;
                  } catch (IOException var8) {
                     var10000 = var8;
                     var10001 = false;
                  }
               } else {
                  try {
                     this.buildConnection(connectTimeout, readTimeout, writeTimeout, var7);
                     continue;
                  } catch (IOException var9) {
                     var10000 = var9;
                     var10001 = false;
                  }
               }
            }

            IOException connectionSpecs2 = var10000;
            Util.closeQuietly(this.socket);
            Util.closeQuietly(this.rawSocket);
            this.socket = null;
            this.rawSocket = null;
            this.source = null;
            this.sink = null;
            this.handshake = null;
            this.protocol = null;
            if (var6 == null) {
               var6 = new RouteException.<init>(connectionSpecs2);
            } else {
               var6.addConnectException(connectionSpecs2);
            }

            if (!connectionRetryEnabled || !var7.connectionFailed(connectionSpecs2)) {
               throw var6;
            }
         }

      }
   }

   public Route route() {
      return this.route;
   }

   public void cancel() {
      Util.closeQuietly(this.rawSocket);
   }

   public Socket socket() {
      return this.socket;
   }

   public boolean isHealthy(boolean doExtensiveChecks) {
      if (!this.socket.isClosed() && !this.socket.isInputShutdown() && !this.socket.isOutputShutdown()) {
         if (this.framedConnection != null) {
            return true;
         } else if (doExtensiveChecks) {
            RealConnection var10000;
            boolean var10001;
            int var10002;
            RealConnection var23;
            try {
               var10000 = this;
               var23 = this;
               var10002 = this.socket.getSoTimeout();
            } catch (SocketTimeoutException var19) {
               var10001 = false;
               return true;
            } catch (IOException var20) {
               var10001 = false;
               return false;
            }

            int doExtensiveChecks1 = var10002;

            boolean var22;
            try {
               var23.socket.setSoTimeout(1);
               var22 = var10000.source.exhausted();
            } catch (Throwable var18) {
               Throwable var2 = var18;

               try {
                  this.socket.setSoTimeout(doExtensiveChecks1);
                  throw var2;
               } catch (SocketTimeoutException var12) {
                  var10001 = false;
                  return true;
               } catch (IOException var13) {
                  var10001 = false;
                  return false;
               }
            }

            if (var22) {
               var22 = false;

               try {
                  this.socket.setSoTimeout(doExtensiveChecks1);
                  return var22;
               } catch (SocketTimeoutException var14) {
                  var10001 = false;
                  return true;
               } catch (IOException var15) {
                  var10001 = false;
               }
            } else {
               var22 = true;

               try {
                  this.socket.setSoTimeout(doExtensiveChecks1);
                  return var22;
               } catch (SocketTimeoutException var16) {
                  var10001 = false;
                  return true;
               } catch (IOException var17) {
                  var10001 = false;
               }
            }

            return false;
         } else {
            return true;
         }
      } else {
         return false;
      }
   }

   public void onStream(FramedStream stream) {
      stream.close(ErrorCode.REFUSED_STREAM);
   }

   public void onSettings(FramedConnection connection) {
      this.allocationLimit = connection.maxConcurrentStreams();
   }

   public Handshake handshake() {
      return this.handshake;
   }

   public boolean isMultiplexed() {
      return this.framedConnection != null;
   }

   public Protocol protocol() {
      if (this.framedConnection == null) {
         Protocol var1;
         if ((var1 = this.protocol) == null) {
            var1 = Protocol.HTTP_1_1;
         }

         return var1;
      } else {
         return this.framedConnection.getProtocol();
      }
   }

   public String toString() {
      StringBuilder var1 = (new StringBuilder()).append("Connection{").append(this.route.address().url().host()).append(":").append(this.route.address().url().port()).append(", proxy=").append(this.route.proxy()).append(" hostAddress=").append(this.route.socketAddress()).append(" cipherSuite=");
      Handshake var2;
      Object var3;
      if ((var2 = this.handshake) != null) {
         var3 = var2.cipherSuite();
      } else {
         var3 = "none";
      }

      return var1.append(var3).append(" protocol=").append(this.protocol).append('}').toString();
   }
}
